@@ -1,7 +1,7 @@
 package App::rimetadb;
 
 our $DATE = '2016-12-10'; # DATE
-our $VERSION = '0.19'; # VERSION
+our $VERSION = '0.20'; # VERSION
 
 use 5.010001;
 use strict;
@@ -640,12 +640,13 @@ sub packages {
 
     if (length $q) {
         push @wheres, "(name LIKE ? OR dist LIKE ? OR extra LIKE ?)";
-        push @binds, $q, $q, $q;
+        push @binds, "%$q%", "%$q%", "%$q%";
     }
 
     my $sth = $dbh->prepare(
-        "SELECT name,summary,dist,mtime,extra FROM package ORDER by name".
-            (@wheres ? " WHERE ".join(" AND ", @wheres) : "")
+        "SELECT name,summary,dist,mtime,extra FROM package".
+            (@wheres ? " WHERE ".join(" AND ", @wheres) : "").
+            " ORDER by name"
     );
     $sth->execute(@binds);
 
@@ -684,7 +685,7 @@ sub functions {
 
     if (length $q) {
         push @wheres, "(package LIKE ? OR name LIKE ? OR dist LIKE ? OR extra LIKE ?)";
-        push @binds, $q, $q, $q, $q;
+        push @binds, "%$q%", "%$q%", "%$q%", "%$q%";
     }
 
     if (defined $args{package}) {
@@ -693,8 +694,9 @@ sub functions {
     }
 
     my $sth = $dbh->prepare(
-        "SELECT package,name,summary,dist,mtime,extra FROM function ORDER by package,name".
-            (@wheres ? " WHERE ".join(" AND ", @wheres) : "")
+        "SELECT package,name,summary,dist,mtime,extra FROM function".
+            (@wheres ? " WHERE ".join(" AND ", @wheres) : "").
+            " ORDER by package,name"
     );
     $sth->execute(@binds);
 
@@ -717,6 +719,11 @@ $SPEC{arguments} = {
         %args_query,
         %args_package,
         %args_function,
+        type => {
+            summary => 'Select arguments with specific type only',
+            schema => 'str*',
+            tags => ['category:filtering'],
+        },
     },
 };
 sub arguments {
@@ -736,7 +743,7 @@ sub arguments {
 
     if (length $q) {
         push @wheres, "(package LIKE ? OR name LIKE ? OR dist LIKE ? OR extra LIKE ?)";
-        push @binds, $q, $q, $q, $q;
+        push @binds, "%$q%", "%$q%", "%$q%", "%$q%";
     }
 
     if (defined $args{package}) {
@@ -750,14 +757,16 @@ sub arguments {
     }
 
     my $sth = $dbh->prepare(
-        "SELECT package,name AS function,metadata FROM function ORDER by package,name".
-            (@wheres ? " WHERE ".join(" AND ", @wheres) : "")
+        "SELECT package,name AS function,metadata FROM function".
+            (@wheres ? " WHERE ".join(" AND ", @wheres) : "").
+            " ORDER by package,name"
     );
     $sth->execute(@binds);
 
     while (my $row0 = $sth->fetchrow_hashref) {
         my $meta = _json->decode(delete $row0->{metadata});
         if ($meta->{args}) {
+          ARG:
             for my $arg (sort keys %{ $meta->{args} }) {
                 my $argspec = $meta->{args}{$arg};
                 my $row = {%$row0};
@@ -768,6 +777,10 @@ sub arguments {
                 $row->{req} = $argspec->{req};
                 $row->{pos} = $argspec->{pos};
                 $row->{greedy} = $argspec->{greedy};
+                if (defined $args{type}) {
+                    next ARG unless defined($row->{schema_type}) &&
+                        $args{type} eq $row->{schema_type};
+                }
                 if ($args{detail}) {
                     push @rows, $row;
                 } else {
@@ -804,13 +817,45 @@ sub stats {
     [200, "OK", \%stats];
 }
 
+$SPEC{function_stats} = {
+    v => 1.1,
+    summary => 'Show some statistics on functions',
+    args => {
+        %args_common,
+    },
+};
+sub function_stats {
+    my %args = @_;
+
+    my ($res, $dbh) = _connect_db(\%args);
+    return $res unless $res->[0] == 200;
+
+    my @rows;
+    my @columns = qw(package name num_args);
+    my @wheres;
+    my @binds;
+
+    my $sth = $dbh->prepare(
+        "SELECT package,name,metadata FROM function".
+            (@wheres ? " WHERE ".join(" AND ", @wheres) : "").
+            " ORDER by package,name"
+    );
+    $sth->execute(@binds);
+
+    while (my $row = $sth->fetchrow_hashref) {
+        my $meta = _json->decode(delete $row->{metadata});
+        $row->{num_args} = keys %{ $meta->{args} // {} };
+        push @rows, $row;
+    }
+
+    [200, "OK", \@rows, {'table.fields'=>\@columns}];
+}
+
 $SPEC{argument_stats} = {
     v => 1.1,
     summary => 'Show statistics on function arguments',
     args => {
         %args_common,
-        #%args_query,
-        #%args_query_detail,
     },
 };
 sub argument_stats {
@@ -916,7 +961,7 @@ App::rimetadb - Manage a Rinci metadata database
 
 =head1 VERSION
 
-This document describes version 0.19 of App::rimetadb (from Perl distribution App-rimetadb), released on 2016-12-10.
+This document describes version 0.20 of App::rimetadb (from Perl distribution App-rimetadb), released on 2016-12-10.
 
 =head1 SYNOPSIS
 
@@ -1001,6 +1046,10 @@ DBI connection password.
 
 =item * B<query> => I<str>
 
+=item * B<type> => I<str>
+
+Select arguments with specific type only.
+
 =item * B<user> => I<str>
 
 DBI connection user.
@@ -1064,6 +1113,47 @@ that contains extra information.
 Return value:  (any)
 
 
+=head2 function_stats(%args) -> [status, msg, result, meta]
+
+Show some statistics on functions.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<dsn> => I<str>
+
+DBI connection DSN.
+
+If not specified, will default to C<dbd:SQLite:$HOME/rimeta.db> where C<$HOME> is
+user's home directory.
+
+Note: has been tested with MySQL and SQLite only.
+
+=item * B<password> => I<str>
+
+DBI connection password.
+
+=item * B<user> => I<str>
+
+DBI connection user.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
 =head2 functions(%args) -> [status, msg, result, meta]
 
 List functions.
@@ -1389,7 +1479,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/App-rimeta
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-App-UpdateRinciMetadataDb>.
+Source repository is at L<https://github.com/perlancar/perl-App-rimetadb>.
 
 =head1 BUGS
 
